/*
 * ProxyFilter.java
 */

package org.grizzly.tunnel;

import com.sun.grizzly.CallbackHandler;
import com.sun.grizzly.ConnectorHandler;
import com.sun.grizzly.Context;
import com.sun.grizzly.Context.*;
import com.sun.grizzly.Controller;
import com.sun.grizzly.Controller.*;
import com.sun.grizzly.IOEvent;
import com.sun.grizzly.ProtocolFilter;
import com.sun.grizzly.filter.ReadFilter;
import com.sun.grizzly.util.OutputWriter;
import com.sun.grizzly.util.WorkerThread;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SelectableChannel;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Alexey Stashok
 */
public class ProxyFilter implements ProtocolFilter {
    private String targetHost;
    private int targetPort;
    
    private Map<SelectableChannel, ConnectorHandler> serverToClient =
            new HashMap<SelectableChannel, ConnectorHandler>();
    private Map<ConnectorHandler, SelectableChannel> clientToServer =
            new HashMap<ConnectorHandler, SelectableChannel>();
    
    private ReadFilter readFilter = new ReadFilter();
    
    public ProxyFilter(String targetHost, int targetPort) {
        this.targetHost = targetHost;
        this.targetPort = targetPort;
    }
    
    public boolean execute(Context context) throws IOException {
        try {
            return processServerRead(context);
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        return false;
    }
    
    public boolean postExecute(Context context) throws IOException {
        if (context.getKeyRegistrationState() == KeyRegistrationState.CANCEL){
            closeByServer(context);
        } else if (context.getKeyRegistrationState() == KeyRegistrationState.REGISTER){
            //@TODO key should be registered to certain protocol's SelectorHandler
            context.getController().registerKey(context.getSelectionKey(),
                    SelectionKey.OP_READ, Protocol.TCP);
        }
        return true;
    }
    
    private boolean processServerRead(Context context) throws IOException {
        if (readFilter.execute(context)) {
            SelectableChannel channel = context.getSelectionKey().channel();
            ConnectorHandler handler = serverToClient.get(channel);
            if (handler == null) {
                handler = context.getController().acquireConnectorHandler(Protocol.TCP);
                handler.setCallbackHandler(new ProxyCallbackHandler(this, handler));
                handler.connect(new InetSocketAddress(targetHost, targetPort));
                serverToClient.put(channel, handler);
                clientToServer.put(handler, channel);
            }
            
            final WorkerThread workerThread = ((WorkerThread)Thread.currentThread());
            ByteBuffer buffer = workerThread.getByteBuffer();
            buffer.flip();
            if (buffer.hasRemaining()) {
                try {
                    OutputWriter.flushChannel((SocketChannel) handler.getUnderlyingChannel(), buffer);
                } finally {
                    buffer.clear();
                }
            }
            
            return true;
        } else {
            return false;
        }
    }
    
    private boolean processServerWrite(Context context) {
        return false;
    }
    
    SocketChannel getServerChannel(ConnectorHandler connectorHandler) {
        return (SocketChannel) clientToServer.get(connectorHandler);
    }
    
    void closeByServer(Context context) {
        Controller controller = context.getController();
        ConnectorHandler handler = serverToClient.remove(context.getSelectionKey().channel());
        if (handler != null) {
            clientToServer.remove(handler);
            try {
                handler.close();
            } catch (IOException ex) {
            }
            
            controller.releaseConnectorHandler(handler);
        }
        
        controller.cancelKey(context.getSelectionKey());
    }
    
    void closeByClient(Context context, ConnectorHandler connectorHandler) {
        Controller controller = context.getController();
        SelectableChannel channel = clientToServer.remove(connectorHandler);
        if (channel != null) {
            serverToClient.remove(channel);
            try {
                channel.close();
            } catch (IOException ex) {
            }
            
            controller.releaseConnectorHandler(connectorHandler);
        }

        try {
            connectorHandler.close();
        } catch (IOException ex) {
        }
    }
}

