/*
 * TunnelListener.java
 */

package org.grizzly.tunnel;

import com.sun.grizzly.Controller;
import com.sun.grizzly.DefaultProtocolChain;
import com.sun.grizzly.DefaultProtocolChainInstanceHandler;
import com.sun.grizzly.ProtocolChain;
import com.sun.grizzly.ProtocolFilter;
import com.sun.grizzly.TCPSelectorHandler;
import com.sun.grizzly.filter.ReadFilter;
import java.io.IOException;

/**
 * @author Alexey Stashok
 */
public class TunnelListener implements Runnable {
    private boolean isStopped;
    private int port;
    private ProtocolFilter proxyFilter;
    
    private Controller controller;
    
    public TunnelListener(int port, ProtocolFilter proxyFilter) {
        this.port = port;
        this.proxyFilter = proxyFilter;
    }
    
    public void start() {
        new Thread(this).start();
    }
    
    public void stop() {
        if (!isStopped) {
            isStopped = true;
            try {
                controller.stop();
            } catch (IOException ex) {
            }
        }
    }
    
    public void run() {
        Controller controller = createController();
        try {
            controller.start();
        } catch(IOException e) {
            e.printStackTrace();
        } finally {
            if (!isStopped) {
                stop();
            }
        }
    }
    
    private Controller createController() {
        TCPSelectorHandler selectorHandler = new TCPSelectorHandler();
        selectorHandler.setPort(port);
        
        final Controller controller = new Controller();
        
        controller.setSelectorHandler(selectorHandler);
        
        controller.setProtocolChainInstanceHandler(
                new DefaultProtocolChainInstanceHandler(){
            public ProtocolChain poll() {
                ProtocolChain protocolChain = protocolChains.poll();
                if (protocolChain == null){
                    protocolChain = new DefaultProtocolChain();
                    protocolChain.addFilter(proxyFilter);
                }
                return protocolChain;
            }
        });
        
        return controller;
    }
}
